/******************************************************************************
  @header BDBDatabaseConfig
 
  @availability OS X, GNUstep
  @copyright (C) 2004, 2005, 2006 Oliver Langer
 
  Author: Oliver Langer

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA

  <pre>
  -------------------------------------------------------------------------
  Modification history

  09.12.2004 ola     initial version
  22.08.2006 ola     license changed
  -------------------------------------------------------------------------
  </pre>
******************************************************************************/

#include "BDBDatabaseConfig.h"

#include <BDB/BDBExceptions.h>

@implementation BDBDatabaseConfig


- init {
  int ret;
  [ super init ];
  
  openFlags = 0;
  generalConfigFlags = 0;

  ret = db_create(&dbp, NULL, 0);
  if (ret != 0) {
    // todo handle this error
  }
  
  return self;
}


- configureDatabase {
  dbp->set_flags( dbp, self->generalConfigFlags );
  return self;
}


- (BDBDatabaseType) databaseType {
  return databaseType;
}


- (DB*) db {
  [self configureDatabase];
  return dbp;
}


- (u_int32_t) generalConfigFlags {
  return self->generalConfigFlags;
}


- (u_int32_t) openFlags {
  return openFlags;
}


- (id) setAllowCreate: (BOOL) enable {
  if( enable ) {
    self->openFlags |= DB_CREATE;
  } else {
    self->openFlags &= ~DB_CREATE;
  }
  return self;
}


- setBTreeRecordNumbering: (BOOL) recordNumbering {
  if( recordNumbering ) {
    self->generalConfigFlags |= DB_RECNUM;
  } else {
    self->generalConfigFlags &= ~DB_RECNUM;
  }
  return self;
}


- (id) setDatabaseType: (BDBDatabaseType) type {
  databaseType = type;
  
  return self;
}


- setAllowDuplicates: (BOOL) duplicatesAllowed {
  if( duplicatesAllowed ) {
    self->generalConfigFlags |= DB_DUP;
  } else {
    self->generalConfigFlags &= ~DB_DUP;
  }

  return self;
}


- setAllocSortedDuplicates: (BOOL) allow {
  if( allow ) {
    self->generalConfigFlags |= DB_DUPSORT;
  } else {
    self->generalConfigFlags &= ~DB_DUPSORT;
  }
  
  return self;
}


- setRecordRenumbering: (BOOL) renumber {
  if( renumber ) {
    self->generalConfigFlags |= DB_RENUMBER;
  } else {
    self->generalConfigFlags &= ~DB_RENUMBER;
  }
  return self;
}
@end
